@@ -28,8 +28,8 @@
 #define EX_SIG 128
 #define EX_SIGINT (EX_SIG + SIGINT)
 
-#define tty_putp(capability) do {					\
-	if (tputs(capability, 1, tty_putc) == ERR)			\
+#define tty_putp(capability, fatal) do {				\
+	if (tputs(capability, 1, tty_putc) == ERR && fatal)		\
 		errx(1, #capability ": unknown terminfo capability");	\
 	} while (0)
 
@@ -279,13 +279,13 @@ selected_choice(void)
 	int	selection = 0;
 	int	yscroll = 0;
 
-	cursor_position = strlen(query);
+	cursor_position = query_length;
 
 	filter_choices();
 
 	for (;;) {
-		tty_putp(cursor_invisible);
-		tty_putp(carriage_return);	/* move cursor to first column */
+		tty_putp(cursor_invisible, 0);
+		tty_putp(carriage_return, 1);	/* move cursor to first column */
 		if (cursor_position >= xscroll + columns)
 			xscroll = cursor_position - columns + 1;
 		if (cursor_position < xscroll)
@@ -307,23 +307,24 @@ selected_choice(void)
 			 */
 			if (tty_putc('\n') == EOF)
 				err(1, "tty_putc");
-			tty_putp(clr_eos);
-			tty_putp(tparm(parm_up_cursor, choices_count + 1));
+			tty_putp(clr_eos, 1);
+			tty_putp(tparm(parm_up_cursor, choices_count + 1), 1);
 		} else {
 			tty_putp(tparm(parm_up_cursor,
 				    choices_count < choices_lines
-				    ? choices_count : choices_lines));
+				    ? choices_count : choices_lines), 1);
 		}
-		tty_putp(carriage_return);	/* move cursor to first column */
+		tty_putp(carriage_return, 1);	/* move cursor to first column */
 		for (i = j = 0; i < cursor_position; j++)
-			while (isu8cont(query[++i]));
+			while (isu8cont(query[++i]))
+				continue;
 		if (j > 0)
 			/*
 			 * parm_right_cursor interprets 0 as 1, therefore only
 			 * move the cursor if the position is non zero.
 			 */
-			tty_putp(tparm(parm_right_cursor, j));
-		tty_putp(cursor_normal);
+			tty_putp(tparm(parm_right_cursor, j), 1);
+		tty_putp(cursor_normal, 0);
 		fflush(tty_out);
 
 		key = get_key(buf, sizeof(buf), &length);
@@ -340,7 +341,8 @@ selected_choice(void)
 			if (cursor_position > 0) {
 				for (length = 1;
 				    isu8cont(query[cursor_position - length]);
-				    length++);
+				    length++)
+					continue;
 				delete_between(
 				    query,
 				    query_length,
@@ -356,7 +358,8 @@ selected_choice(void)
 			if (cursor_position < query_length) {
 				for (length = 1;
 				    isu8cont(query[cursor_position + length]);
-				    length++);
+				    length++)
+					continue;
 				delete_between(
 				    query,
 				    query_length,
@@ -393,7 +396,8 @@ selected_choice(void)
 				break;
 
 			for (word_position = cursor_position;;) {
-				while (isu8cont(query[--word_position]));
+				while (isu8cont(query[--word_position]))
+					continue;
 				if (word_position < 1)
 					break;
 				if (query[word_position] != ' '
@@ -432,11 +436,13 @@ selected_choice(void)
 			break;
 		case LEFT:
 			while (cursor_position > 0
-			    && isu8cont(query[--cursor_position]));
+			    && isu8cont(query[--cursor_position]))
+				continue;
 			break;
 		case RIGHT:
 			while (cursor_position < query_length
-			    && isu8cont(query[++cursor_position]));
+			    && isu8cont(query[++cursor_position]))
+				continue;
 			break;
 		case PAGE_DOWN:
 			if (selection + choices_lines < choices_count)
@@ -554,7 +560,8 @@ min_match(const char *string, size_t offset, ssize_t *start, ssize_t *end)
 		return 0;
 
 	for (;;) {
-		for (e++, q++; isu8cont(*q); e++, q++);
+		for (e++, q++; isu8cont(*q); e++, q++)
+			continue;
 		if (*q == '\0')
 			break;
 		if ((e = strcasechr(e, q)) == NULL)
@@ -622,7 +629,7 @@ tty_init(void)
 	choices_lines = lines - 1; /* available lines, minus query line */
 
 	if (use_alternate_screen)
-		tty_putp(enter_ca_mode);
+		tty_putp(enter_ca_mode, 0);
 
 	signal(SIGINT, handle_sigint);
 }
@@ -646,11 +653,11 @@ tty_restore(void)
 	tcsetattr(fileno(tty_in), TCSANOW, &original_attributes);
 	fclose(tty_in);
 
-	tty_putp(carriage_return);	/* move cursor to first column */
-	tty_putp(clr_eos);
+	tty_putp(carriage_return, 1);	/* move cursor to first column */
+	tty_putp(clr_eos, 1);
 
 	if (use_alternate_screen)
-		tty_putp(exit_ca_mode);
+		tty_putp(exit_ca_mode, 0);
 
 	fclose(tty_out);
 }
@@ -664,11 +671,11 @@ print_line(const char *string, size_t length, int so, ssize_t ulso, ssize_t uleo
 	int	non_printable = 0;
 
 	if (so)
-		tty_putp(enter_standout_mode);
+		tty_putp(enter_standout_mode, 1);
 
 	for (col = i = 0; i < length && col < columns; i++) {
 		if (i == (size_t)ulso)
-			tty_putp(enter_underline_mode);
+			tty_putp(enter_underline_mode, 1);
 
 		/*
 		 * Count the number of outputted ANSI escape sequences
@@ -707,14 +714,14 @@ print_line(const char *string, size_t length, int so, ssize_t ulso, ssize_t uleo
 		}
 
 		if (i + 1 == (size_t)uleo)
-			tty_putp(exit_underline_mode);
+			tty_putp(exit_underline_mode, 1);
 	}
 	for (col -= non_printable; col < columns; col++)
 		if (tty_putc(' ') == EOF)
 			err(1, "tty_putc");
 
 	if (so)
-		tty_putp(exit_standout_mode);
+		tty_putp(exit_standout_mode, 1);
 }
 
 /*
@@ -732,7 +739,7 @@ print_choices(int offset, int selection)
 	     (size_t)i < choices.length
 	     && (query_length == 0 || choice->score > 0);
 	     choice++, i++) {
-		if (i - offset < lines - 1)
+		if (i - offset < choices_lines)
 			print_line(choice->string, choice->length,
 			    i == selection, choice->match_start,
 			    choice->match_end);
